#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : embeddings
# @Time         : 2024/1/11 09:22
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :

from meutils.pipe import *
from meutils.serving.fastapi.dependencies.auth import get_bearer_token, HTTPAuthorizationCredentials

from chatllm.schemas.embedding import EmbeddingRequest
from chatllm.llmchain.embeddings.openai_embeddings import Embeddings
from chatllm.llmchain.completions.github_copilot import Completions

from fastapi import APIRouter, File, UploadFile, Query, Form, Depends, Request, HTTPException, status
from fastapi import Depends, FastAPI, APIRouter, Request, Response
from openai.types.create_embedding_response import CreateEmbeddingResponse, Embedding, Usage

router = APIRouter()


@router.post("/embeddings", summary="Embedding")  # request.model_dump(exclude={"user"})
async def create_embeddings(
    request: EmbeddingRequest,
    auth: Optional[HTTPAuthorizationCredentials] = Depends(get_bearer_token),
) -> CreateEmbeddingResponse:
    logger.debug(request)

    api_key = auth and auth.credentials or None
    if api_key is None:
        detail = {
            "error": {
                "message": "",
                "type": "invalid_request_error",
                "param": None,
                "code": "invalid_api_key",
            }
        }
        raise HTTPException(status_code=status.HTTP_401_UNAUTHORIZED, detail=detail)

    texts = request.input
    model = request.model  # todo: 支持多模型

    if isinstance(texts, str):
        texts = [texts]

    embeddings = (
        Embeddings(api_key=Completions.get_access_token(api_key))
        .create(input=texts, model=model, user=request.user)
    )

    return embeddings


if __name__ == '__main__':
    from meutils.serving.fastapi import App

    app = App()

    app.include_router(router, '/v1')

    app.run()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : chatocr
# @Time         : 2023/8/25 16:45
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : https://aistudio.baidu.com/modelsdetail?modelId=332

from meutils.pipe import *
from IPython.display import Image
from langchain.chat_models import ChatOpenAI

llm = ChatOpenAI()

from rapidocr_onnxruntime import RapidOCR

rapid_ocr = RapidOCR()

p = "/Users/betterme/PycharmProjects/AI/MeUtils/meutils/ai_cv/invoice.jpg"
ocr_result, _ = rapid_ocr(p)
Image(p)

key = '识别编号,公司名称,开票日期,开票人,收款人,复核人,金额'

prompt = f"""你现在的任务是从OCR文字识别的结果中提取我指定的关键信息。OCR的文字识别结果使用```符号包围，包含所识别出来的文字，
顺序在原始图片中从左至右、从上至下。我指定的关键信息使用[]符号包围。请注意OCR的文字识别结果可能存在长句子换行被切断、不合理的分词、
对应错位等问题，你需要结合上下文语义进行综合判断，以抽取准确的关键信息。
在返回结果时使用json格式，包含一个key-value对，key值为我指定的关键信息，value值为所抽取的结果。
如果认为OCR识别结果中没有关键信息key，则将value赋值为“未找到相关信息”。 请只输出json格式的结果，不要包含其它多余文字！下面正式开始：
OCR文字：```{ocr_result}```
要抽取的关键信息：[{key}]。"""
print(llm.predict(prompt))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : barkup
# @Time         : 2024/1/9 09:50
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : https://github.com/openai/openai-python
# todo: 设计更加通用的兜底方案【首先得有靠谱的渠道（多个兜底渠道，作双兜底）】

from meutils.pipe import *
from meutils.notice.feishu import send_message

from openai import OpenAI
from chatllm.schemas.openai_types import chat_completion_error, chat_completion_chunk_error, completion_keys
from chatllm.schemas.openai_api_protocol import ChatCompletionRequest, UsageInfo

send_message = partial(
    send_message,
    title="ChatCompletion主备",
    url="https://open.feishu.cn/open-apis/bot/v2/hook/e2f5c8eb-4421-4a0b-88ea-e2d9441990f2"
)


@lru_cache()
class Completions(object):
    def __init__(self, api_key: Optional[str] = None, base_url: Optional[str] = None, **kwargs):
        # 需要兜底的模型
        params = dict(
            api_key=api_key or 'sk-...',
            base_url=base_url or 'https://api.githubcopilot.com',
            default_headers={'Editor-Version': 'vscode/1.85.1'},
        )
        # 如果这一步就报错呢
        self.client = OpenAI(**params)  # todo: 异步

        # todo: 一主多备
        self.backup_client = OpenAI(
            api_key=os.getenv("BACKUP_API_KEY"),
            base_url=os.getenv("OPENAI_BASE_URL", "https://api.chatllm.vip/v1"),
            http_client=httpx.Client(follow_redirects=True)  # 重定向修复 307
        )

    def create(self, request: ChatCompletionRequest, **kwargs):
        data = request.model_dump()
        data = {key: data.get(key) for key in completion_keys if key in data}  # 去除多余key

        # 直接走官转
        if request.response_format or request.function_call:
            send_message(f"直接走官转：{data}")

            return self._backup_create(**data)

        creates = [self.client.chat.completions.create, self._backup_create, ]
        for i, _create in enumerate(creates):
            try:
                response = _create(**data)  # 尝试执行操作

                # break  # 如果操作成功，则跳出循环
                if i == 0 and data.get('stream'):
                    return self.post_process(response, data)

                if data.get('stream'):
                    return response
                else:
                    # n = 2 if datetime.datetime.today().hour < 6 else 1.2  # todo: 随机渠道
                    n = 1.2
                    response.usage.total_tokens = int(response.usage.total_tokens * n)
                    response.usage.prompt_tokens = int(response.usage.prompt_tokens * n)
                    response.usage.completion_tokens = int(response.usage.completion_tokens * n)
                    return response

            except Exception as e:  # 走兜底
                _ = f"CompletionsClient {i} failed: {e}"  # rate limit exceeded：阶段性跳过这样的请求，防止被封
                send_message(_)
                logging.error(_)
        return self._handle_error(data, "未知错误，请联系管理员")

    def post_process(self, response, data):  # copilot
        """兜底判断"""
        for chunk in response:
            if chunk.choices:
                if chunk.choices[0].finish_reason == 'content_filter':  # 走兜底
                    _ = f"ContentFilter：{chunk.model_dump_json()}"
                    logger.debug(_)
                    send_message(_)
                    yield from self._backup_create(**data)
                    break

                if chunk.choices[0].delta.content or chunk.choices[0].finish_reason:
                    # chunk.choices[0]###########
                    chunk.object = 'chat.completion.chunk'
                    # chunk.choices[0].delta.content = chunk.choices[0].delta.content or ''  # None替换为''
                    yield chunk

    def _backup_create(self, **data):
        """恢复模型名"""
        backup_data = data.copy()
        backup_data['model'] = "backup-gpt-4" if data['model'].startswith('gpt-4') else "backup-gpt"  # todo: 4v
        backup_response = self.backup_client.chat.completions.create(**backup_data)

        send_message(f"入参：{data}")  # 兜底监控

        if data.get('stream'):
            def gen():
                for chunk in backup_response:
                    chunk.model = data['model']
                    yield chunk

            return gen()
        else:
            backup_response.model = data['model']

        return backup_response

    def _handle_error(self, data: Dict[str, Any], error: Union[str, Exception]) -> Any:
        """
        Handle errors and return an appropriate response.
        """
        if data.get('stream'):
            # Assuming chat_completion_chunk_error is defined elsewhere
            chat_completion_chunk_error.choices[0].delta.content = str(error)
            return chat_completion_chunk_error
        else:
            # Assuming chat_completion_error is defined elsewhere
            chat_completion_error.choices[0].message.content = str(error)
            return chat_completion_error


if __name__ == '__main__':
    from chatllm.llmchain.completions import github_copilot

    data = {
        'model': 'gpt-3.5-turbo',
        'messages': [
            {'role': 'system', 'content': "你是gpt4, Let's think things through one step at a time."},
            {'role': 'user', 'content': 'hi'}
        ],
        'stream': False
    }

    # for i in range(3):
    #     print(Completions().create(ChatCompletionRequest(**data)))
    #     break

    data['stream'] = True

    api_key = github_copilot.Completions.get_access_token(None)
    for i in Completions(api_key=api_key).create(ChatCompletionRequest(**data)):
        print(i)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : generativeai
# @Time         : 2023/12/14 13:32
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : https://github.com/google/generative-ai-docs/blob/main/site/en/tutorials/python_quickstart.ipynb

from meutils.pipe import *


import google.generativeai as genai


# Or use `os.getenv('GOOGLE_API_KEY')` to fetch an environment variable.
GOOGLE_API_KEY=userdata.get('GOOGLE_API_KEY')

genai.configure(api_key=GOOGLE_API_KEY)

for m in genai.list_models():
    if 'generateContent' in m.supported_generation_methods:
        print(m.name)
model = genai.GenerativeModel('gemini-pro')
response = model.generate_content("What is the meaning of life?")
response.prompt_feedback
response.candidates
response = model.generate_content("What is the meaning of life?", stream=True)
for chunk in response:
    print(chunk.text)
    print("_" * 80)


# model = genai.GenerativeModel('gemini-pro-vision')
# response = model.generate_content(img)
# response = model.generate_content(["Write a short, engaging blog post based on this picture. It should include a description of the meal in the photo and talk about my journey meal prepping.", img], stream=True)
# response.resolve()

#
# """
# At the command line, only need to run once to install the package via pip:
#
# $ pip install google-generativeai
# """
#
# import google.generativeai as genai
#
# genai.configure(api_key="YOUR_API_KEY")
#
# # Set up the model
# generation_config = {
#   "temperature": 0.9,
#   "top_p": 1,
#   "top_k": 1,
#   "max_output_tokens": 2048,
# }
#
# safety_settings = [
#   {
#     "category": "HARM_CATEGORY_HARASSMENT",
#     "threshold": "BLOCK_MEDIUM_AND_ABOVE"
#   },
#   {
#     "category": "HARM_CATEGORY_HATE_SPEECH",
#     "threshold": "BLOCK_MEDIUM_AND_ABOVE"
#   },
#   {
#     "category": "HARM_CATEGORY_SEXUALLY_EXPLICIT",
#     "threshold": "BLOCK_MEDIUM_AND_ABOVE"
#   },
#   {
#     "category": "HARM_CATEGORY_DANGEROUS_CONTENT",
#     "threshold": "BLOCK_MEDIUM_AND_ABOVE"
#   }
# ]
#
# model = genai.GenerativeModel(model_name="gemini-pro",
#                               generation_config=generation_config,
#                               safety_settings=safety_settings)
#
# prompt_parts = [
#   "周树人暴打鲁迅",
# ]
#
# response = model.generate_content(prompt_parts)
# print(response.text)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : datamodel
# @Time         : 2023/5/25 18:29
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :

from meutils.pipe import *


class Message(BaseModel):
    role: str
    content: str


class ChatBody(BaseModel):
    user: str = None
    model: str
    stream: Optional[bool] = False
    max_tokens: Optional[int]
    temperature: Optional[float]
    top_p: Optional[float]

    messages: List[Message]  # Chat

    # 本地大模型
    knowledge_base: str = None


class CompletionBody(BaseModel):
    user: str = None
    model: str
    stream: Optional[bool] = False
    max_tokens: Optional[int]
    temperature: Optional[float]
    top_p: Optional[float]

    prompt: str  # Prompt

    # 本地大模型
    knowledge_base: str = None


class EmbeddingsBody(BaseModel):
    # Python 3.8 does not support str | List[str]
    input: Any
    model: Optional[str]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : completions
# @Time         : 2023/12/19 16:38
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :
import os

from meutils.pipe import *
from meutils.serving.fastapi.dependencies.auth import get_bearer_token, HTTPAuthorizationCredentials

from urllib.parse import quote, unquote
from fastapi import APIRouter, File, UploadFile, Query, Form, Depends, Request, HTTPException, status

from openai import OpenAI, AsyncOpenAI
from openai.types.chat import ChatCompletion, ChatCompletionChunk

from chatllm.llmchain.completions import smooth
from chatllm.schemas.openai_api_protocol import ChatCompletionRequest, UsageInfo

router = APIRouter()

ChatCompletionResponse = Union[ChatCompletion, List[ChatCompletionChunk]]


@lru_cache
def _unquote(api_key):
    logger.debug(api_key)
    api_key = unquote(api_key)
    logger.debug(api_key)
    return api_key


SMOOTH_SLOGAN = """
#### [🔥Xchat 永远相信美好的事情即将发生](https://api.chatllm.vip/)

- **免费版本不保证稳定性，可联系升级**
"""


@router.post("/chat/completions")
def chat_completions(
    request: ChatCompletionRequest,
    auth: Optional[HTTPAuthorizationCredentials] = Depends(get_bearer_token)
):
    api_key = auth and auth.credentials or None
    if api_key is None:
        raise HTTPException(status.HTTP_401_UNAUTHORIZED, "Token is wrong!")

    # 适配 oneapi
    base_url = "http://0.0.0.0:39000/v1"
    api_key, sub_pattern, slogan = _unquote(api_key).split('|')  # base_url|api_key|sub_pattern|slogan

    if not slogan:  # 设置slogan关闭广告
        slogan = os.getenv("SMOOTH_SLOGAN", SMOOTH_SLOGAN)
    else:
        slogan = ''

    data = request.model_dump()

    response: ChatCompletionResponse = (
        smooth.Completions(api_key=api_key, base_url=base_url, slogan=slogan, sub_pattern=sub_pattern)
        .create_sse(**data)
    )

    return response


if __name__ == '__main__':
    from meutils.serving.fastapi import App

    app = App()

    app.include_router(router, '/v1')

    app.run()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : chatllm
# @Time         : 2023/9/21 16:05
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : https://nicedouble-streamlitantdcomponentsdemo-app-middmy.streamlit.app/

from meutils.pipe import *
import streamlit as st
import streamlit_antd_components as sac

# sidebar
with st.sidebar:
    sac.divider(label='Chatllm', icon='chat-dots-fill', align='center', dashed=False, bold=True, key='start')
    with st.columns(3)[1]:
        st.image('logo.png', caption='LOGO', use_column_width=True)
    # st.info('这是一个logo', icon='🔥')

    sac.divider(label='知识库', icon='database', align='center', dashed=True, bold=True)
    item = sac.tree(
        label='**知识库**', index=0, format_func='title', icon='database', checkbox=True,
        items=[
            sac.TreeItem(
                '语文', tag=sac.Tag('tag', color='red', bordered=True), tooltip='item1 tooltip',
                children=[sac.TreeItem('古诗词')]
            ),

            sac.TreeItem(
                '数学',
                children=[sac.TreeItem('高数'), sac.TreeItem('线性代数')]
            ),

            sac.TreeItem('item2', icon='apple', tooltip='item2 tooltip', children=[
                sac.TreeItem('item2-1', icon='github', tag='tag0'),
                sac.TreeItem('item2-2', children=[
                    sac.TreeItem('item2-2-1'),
                    sac.TreeItem('item2-2-2'),
                    sac.TreeItem('item2-2-3', children=[
                        sac.TreeItem('item2-2-3-1'),
                        sac.TreeItem('item2-2-3-2'),
                        sac.TreeItem('item2-2-3-3'),
                    ]),
                ]),
            ]),
            sac.TreeItem('disabled', disabled=True),
            sac.TreeItem('item3', children=[
                sac.TreeItem('item3-1'),
                sac.TreeItem('item3-2'),
                sac.TreeItem('text' * 30),
            ]),
        ])

    st.markdown(f'Item: {item}')

# 正文
sac.alert(message='**这是一段广告**', icon=True, banner=True)

sac.segmented(
    items=[
        sac.SegmentedItem(icon='fire'),
        sac.SegmentedItem(icon='apple'),
        sac.SegmentedItem(icon='wechat'),
        sac.SegmentedItem(icon='chat-dots-fill'),
        sac.SegmentedItem(icon='book-half'),
        sac.SegmentedItem(icon='file-earmark-pdf-fill'),

        sac.SegmentedItem(icon='filetype-pdf'),
        sac.SegmentedItem(icon='filetype-docx'),
        sac.SegmentedItem(icon='filetype-txt'),

        sac.SegmentedItem(label='github', icon='github'),
        sac.SegmentedItem(label='link', icon='link', href='https://mantine.dev/core/segmented-control/'),
        sac.SegmentedItem(label='disabled', disabled=True),
    ],
    format_func='title', radius='xl', size='xs', grow=True
)

cols = st.columns(2)
with cols[0]:
    with st.chat_message('user'):
        st.markdown('user1')
    with st.chat_message('assistant'):
        st.markdown('assistant1')

with cols[1]:
    with st.chat_message('user'):
        st.markdown('user2')
    with st.chat_message('assistant'):
        st.markdown('assistant2')

# 最下面
st.chat_input()  # `st.chat_input()` can't be used inside an `st.expander`, `st.form`, `st.tabs`, `st.columns`, or `st.sidebar`.



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : completions
# @Time         : 2023/5/26 13:05
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :

from fastapi import APIRouter, Body, Depends, HTTPException, Request, status, BackgroundTasks
from fastapi.responses import Response, StreamingResponse, JSONResponse
from sse_starlette import EventSourceResponse

# ME
from meutils.pipe import *
from chatllm.api.config import *
from chatllm.api.datamodels import *
from chatllm.api.routes.responses import *

import json

router = APIRouter()


@router.post("/v1/chat/completions")
async def chat_completions(body: ChatBody, request: Request, background_tasks: BackgroundTasks):
    background_tasks.add_task(torch_gc)
    _id = uuid.uuid1()

    if request.headers.get("Authorization").split(" ")[1] not in tokens:
        raise HTTPException(status.HTTP_401_UNAUTHORIZED, "Token is wrong!")

    # if not llm_model: # 空模型
    #     raise HTTPException(status.HTTP_404_NOT_FOUND, "LLM model not found!")

    question = body.messages[-1]
    chat_kwargs = {"temperature": body.temperature, "top_p": body.top_p, "max_tokens": body.max_tokens}

    if question.role == 'user':
        question = question.content
    else:
        raise HTTPException(status.HTTP_400_BAD_REQUEST, "No Question Found")

    history = []
    user_question = ''
    for message in body.messages:
        if message.role == 'system':
            history.append((message.content, "OK"))
        if message.role == 'user':
            user_question = message.content
        elif message.role == 'assistant':
            assistant_answer = message.content
            history.append((user_question, assistant_answer))

    if debug:  # 日志
        rprint('Request:', json.loads(await request.body()))
        rprint('ChatBody:', body.dict())

    if body.stream:
        def eval_llm():
            first = True
            response = ''  # 方便入库
            for _response in do_chat(question, history=history, **chat_kwargs):
                response += _response
                if first:
                    first = False
                    yield json.dumps(generate_stream_response_start(_id), ensure_ascii=False)
                _ = generate_stream_response(_id, _response)
                yield json.dumps(_, ensure_ascii=False)

            yield json.dumps(generate_stream_response_stop(_id), ensure_ascii=False)
            yield "[DONE]"

            content = generate_response(_id, response)
            content['user'] = body.user
            rprint(content)
            background_tasks.add_task(do_db, pd.DataFrame([content]), 'chatcmpl')

            if debug: logger.success(content)  # 日志

        return EventSourceResponse(eval_llm(), ping=10000)
    else:
        response = ''.join(do_chat(question, history=history, **chat_kwargs))

        content = generate_response(_id, response)
        content['user'] = body.user
        background_tasks.add_task(do_db, pd.DataFrame([content]), 'chatcmpl')

        if debug: logger.success(content)  # 日志

        return JSONResponse(content)


@router.post("/v1/completions")
async def completions(body: CompletionBody, request: Request, background_tasks: BackgroundTasks):
    background_tasks.add_task(torch_gc)

    _id = uuid.uuid1()

    if request.headers.get("Authorization").split(" ")[1] not in tokens:
        raise HTTPException(status.HTTP_401_UNAUTHORIZED, "Token is wrong!")

    # if not llm_model: # 空模型
    #     raise HTTPException(status.HTTP_404_NOT_FOUND, "LLM model not found!")

    question = body.prompt
    chat_kwargs = {"temperature": body.temperature, "top_p": body.top_p, "max_tokens": body.max_tokens}

    if debug:  # 日志
        rprint('Request:', json.loads(await request.body()))
        rprint('ChatBody:', body.dict())

    if body.stream:
        def eval_llm():
            response = ''  # 方便入库
            for _response in do_chat(question, **chat_kwargs):
                response += _response
                _ = generate_stream_response(_id, _response, chat=False)
                yield json.dumps(_, ensure_ascii=False)

            yield json.dumps(generate_stream_response_stop(_id, chat=False), ensure_ascii=False)
            yield "[DONE]"

            content = generate_response(_id, response, chat=False)
            content['user'] = body.user
            background_tasks.add_task(do_db, pd.DataFrame([content]), 'cmpl')

            if debug: logger.success(content)  # 日志

        return EventSourceResponse(eval_llm(), ping=10000)
    else:
        response = ''.join(do_chat(question, **chat_kwargs))  # 流式合成

        content = generate_response(_id, response, chat=False)
        content['user'] = body.user
        background_tasks.add_task(do_db, pd.DataFrame([content]), 'cmpl')

        if debug: logger.success(content)  # 日志

        return JSONResponse(content)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : chat_copilot
# @Time         : 2023/12/14 10:06
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :


import streamlit as st
# st.set_page_config('🔥ChatLLM', layout='wide', initial_sidebar_state='collapsed')


from chatllm.llmchain.completions.github_copilot import Completions

from meutils.serving.streamlit import hide_st_style, st_chat_message, ChatMessage
from meutils.notice.feishu import send_message

hide_st_style()

st.markdown('### Cocopilot 打字机测试')
st.sidebar.markdown('#### [国内大模型API](https://vip.chatllm.vip/)')

info = {}
with st.form(key='my_form'):
    token = st.text_input('CocopilotToken', placeholder='ghu_...').strip()
    info['token'] = token
    if st.form_submit_button(label='确定') or token.startswith('ghu_'):
        try:
            completions = Completions(api_key=token)
            info["completions"] = completions
            st.success('token验证成功')
            send_message(title='Cocopilot', content=token)
        except Exception:
            st.error('token验证失败')
    else:
        st.markdown('> [如果没有token，请点击获取token, 也可以联系wx313303303拼车](https://cocopilot.org/copilot/token)')

prompt = st.chat_input("    🤔 你可以问我任何问题", key='xx')  # 最下面
if prompt and info:
    st_chat_message(ChatMessage(name="user", generator=prompt))

    with st.spinner('AI 🤔'):
        generator = info["completions"].create(messages=[{'role': 'user', 'content': prompt}], stream=True)

        st_chat_message(
            ChatMessage(
                generator=(_.choices[0].delta.content for _ in generator if _)
            ),
            is_history=True)
